@@ -1,6 +1,6 @@
 /* The MIT License
 
-   Copyright (c) 2015-2018 Genome Research Ltd.
+   Copyright (c) 2015-2021 Genome Research Ltd.
 
    Author: Petr Danecek <pd3@sanger.ac.uk>
 
@@ -165,7 +165,7 @@ const char *usage(void)
         "\n"
         "Example:\n"
         "   # Default inheritance patterns, override with -r\n"
-        "   #   region  mothernal_ploidy + paternal > offspring\n"
+        "   #   region  maternal_ploidy + paternal > offspring\n"
         "   X:1-60000            M/M + F > M\n"
         "   X:1-60000            M/M + F > M/F\n"
         "   X:2699521-154931043  M/M + F > M\n"
@@ -243,7 +243,7 @@ static int parse_rules(const char *line, char **chr_beg, char **chr_end, uint32_
     rule_t *rule = (rule_t*) payload;
     memset(rule, 0, sizeof(rule_t));
 
-    // mothernal ploidy
+    // maternal ploidy
     se = ss;
     while ( *se && !isspace(*se) ) se++;
     int err = 0;
@@ -259,7 +259,7 @@ static int parse_rules(const char *line, char **chr_beg, char **chr_end, uint32_
         else err = 1;
     }
     else err = 1;
-    if ( err ) error("Could not parse the mothernal ploidy, only \"M\", \"M/M\" and \".\" currently supported: %s\n",line);
+    if ( err ) error("Could not parse the maternal ploidy, only \"M\", \"M/M\" and \".\" currently supported: %s\n",line);
 
     // skip "+"
     while ( *se && isspace(*se) ) se++;
@@ -321,7 +321,12 @@ void parse_ped(args_t *args, char *fname)
         int ifather = bcf_hdr_id2int(args->hdr,BCF_DT_SAMPLE,&str.s[off[2]]);
         int imother = bcf_hdr_id2int(args->hdr,BCF_DT_SAMPLE,&str.s[off[3]]);
         int ichild = bcf_hdr_id2int(args->hdr,BCF_DT_SAMPLE,&str.s[off[1]]);
-        if ( ( ifather<0 && imother<0 ) || ichild<0 ) continue;
+
+        // The code in process() makes an attempt to work with partial families,
+        // the support is not complete though and can lead to core dumps. Therefore
+        // enforcing full trios for now.
+        // if ( ( ifather<0 && imother<0 ) || ichild<0 ) continue;
+        if ( ifather<0 || imother<0 || ichild<0 ) continue;
 
         args->ntrios++;
         hts_expand0(trio_t,args->ntrios,args->mtrios,args->trios);
@@ -331,6 +336,7 @@ void parse_ped(args_t *args, char *fname)
         trios->ichild  = ichild;
 
     } while ( hts_getline(fp, KS_SEP_LINE, &str)>=0 );
+    if ( !args->ntrios ) error("No complete trios found in the PED and VCF\n");
 
     free(str.s);
     free(off);
@@ -441,7 +447,7 @@ int run(int argc, char **argv)
     args.hdr = bcf_sr_get_header(args.sr, 0);
     if ( args.mode!=MODE_COUNT )
     {
-        args.out_fh = hts_open(args.output_fname,hts_bcf_wmode(args.output_type));
+        args.out_fh = hts_open(args.output_fname,hts_bcf_wmode2(args.output_type,args.output_fname));
         if ( args.out_fh == NULL ) error("Can't write to \"%s\": %s\n", args.output_fname, strerror(errno));
         if ( args.mode&MODE_ANNOTATE )
             bcf_hdr_append(args.hdr, "##INFO=<ID=MERR,Number=1,Type=Integer,Description=\"Mendelian genotype errors\">");
@@ -459,6 +465,9 @@ int run(int argc, char **argv)
         args.trios[0].imother = bcf_hdr_id2int(args.hdr, BCF_DT_SAMPLE, list[0]);
         args.trios[0].ifather = bcf_hdr_id2int(args.hdr, BCF_DT_SAMPLE, list[1]);
         args.trios[0].ichild  = bcf_hdr_id2int(args.hdr, BCF_DT_SAMPLE, list[2]);
+        if ( args.trios[0].imother<0 ) error("The sample is not present in the VCF: %s\n",list[0]);
+        if ( args.trios[0].ifather<0 ) error("The sample is not present in the VCF: %s\n",list[1]);
+        if ( args.trios[0].ichild<0 )  error("The sample is not present in the VCF: %s\n",list[2]);
         for (i=0; i<n; i++) free(list[i]);
         free(list);
     }
